/*
 AD9850 control codet by Dr. David Mills, G7UVW http://www.webshed.org/wiki/G7UVW
 Datasheet at http://www.analog.com/static/imported-files/data_sheets/AD9850.pdf

 Original rotary encoder control program by Richard Visokey, AD7C - www.ad7c.com - revision 2.0 - November 6th, 2013
 
 Arduino rotary encoder code (C)2011 by Ben Buxton (bb@cactii.net) & licenced under GNU GPL version 3
  
 Additional modifications by Robert C. Mazur, VA3ROM http://www.va3rom.com 16 April 2015
*/

/*************************************************************************************************************
 *                                            ARDUINO INCLUDE FILES                                          *
 *************************************************************************************************************/
#include <Wire.h>                           // I2C support library
#include <EEPROM.h>                         // EEPROM support library

#include <LiquidCrystal_I2C.h>              // unremark line if using standard 16x2 LCD with I2C module driver

  LiquidCrystal_I2C lcd(0x27, 16, 2);       // define LCD for I2C address 0x27 & 16x2 display

/*************************************************************************************************************
 *                               USER GLOBAL PROGRAM PARAMETERS (CHANGE AS REQUIRED)                         * 
 *************************************************************************************************************/

// Arduino digital pins to AD9850 DDS module connections (change/uncomment as required)
#define  CLOCK  8                           // pin  8 to word load clock (CLK)
#define  LOAD   9                           // pin  9 to freq update (FQ)
#define  DATA  10                           // pin 10 to serial data load (DATA)
#define  RESET 11                           // pin 11 to reset (RST)

// Arduino digital pins to Kanga UK AD9850 DDS module shield connections
//#define  CLOCK  2                           // pin 2 to word load clock//CLK)
//#define  LOAD   3                           // pin 3 to freq update (FQ//
//#define  DATA   4                           // pin 4 to serial data load (DATA)
//#define  RESET  5                           // pin 5 to reset (RST)

#define DDS_CLOCK 125000000                 // my AD9850 DDS modules XTAL is 125 MHz (change as required)
#define DDS_CORR 0.0                        // parts-per-million (ppm) error only for this DDS module becasue they are all different (change as required)

/*************************************************************************************************************
 *                                         GLOBAL VARIABLE DECLARATIONS                                      *
 *************************************************************************************************************/
#define BUTTON_S1 A0                        // proto-shield board button labelled 'S1' connected analog pin A0 which is digital pin 14
#define RF_PROBE A1                         // AD9850 DDS RF drive level to SA/NE602/612 chip connected to analog pin A1

// create Arduiono DCR/SDR record memory channel structure
#define MAXCHANNELS 26                      // number of used record memory channels

// this data structure is stored in 2K SRAM which is used for data & program variables (don't exceed 64 records!)
struct record {
 
  String label;                             // record channel name (maximum 10 characters for proper LCD display)
  double frequency;                         // record channel frequency in MHz

} channel[MAXCHANNELS + 1];                 // allocate room in dynamic memory

int index = 0;                              // memory channel record index pointer
int tuning = 1;                             // memory channel tuning toggle flag : 1 = tune up : -1 = tune down

// create AD9850 DDS module 32-bit turning word from requested frequency
void AD9850_setfreq(double frequency) {
 
  // convert frequency to 32-bit turning word based on DDS module clock speed & correctiong for clock ppm (parts-per-million) error
  unsigned long tuning_word = (double)(frequency * pow(2, 32)) / (double)(DDS_CLOCK - (DDS_CLOCK * (DDS_CORR / 1e6)));
  
  digitalWrite (LOAD, LOW);                 // enable DDS load of tuning word into data register

  // load 32-bit tuning word in four 8-bit (byte) segments
  shiftOut(DATA, CLOCK, LSBFIRST, tuning_word);
  shiftOut(DATA, CLOCK, LSBFIRST, tuning_word >> 8);
  shiftOut(DATA, CLOCK, LSBFIRST, tuning_word >> 16);
  shiftOut(DATA, CLOCK, LSBFIRST, tuning_word >> 24);
  
  shiftOut(DATA, CLOCK, LSBFIRST, 0x0);    // end of tuning word
  
  digitalWrite (LOAD, HIGH);               // generate new DDS frequency from data loaded into registers

}

// initialize AD9850 DDS module
void AD9850_init() {

  // pulkse all data registers high then low
  digitalWrite(RESET, HIGH);
    digitalWrite(RESET, LOW);
  
  digitalWrite(CLOCK, HIGH);
    digitalWrite(CLOCK, LOW);
  
  digitalWrite(LOAD, HIGH); 
    digitalWrite(LOAD, LOW);
  
  digitalWrite(DATA, HIGH); 
    digitalWrite(DATA, LOW);
  
}

// reset AD9850 DDS module
void AD9850_reset() {
  
  /* reset sequence is:
     CLOCK & LOAD = LOW
     Pulse RESET high then low
     Pulse CLOCK high then low
     Set DATA to ZERO and pulse LOAD */

  // data sheet diagrams show only RESET and CLOCK being used to reset the device but you won't see any output unless you reset LOAD line too

  digitalWrite(CLOCK, LOW);
  digitalWrite(LOAD, LOW);

  digitalWrite(RESET, LOW);
  digitalWrite(RESET, HIGH);                  // pulse RESET
  digitalWrite(RESET, LOW);
 
  digitalWrite(CLOCK, LOW);
  digitalWrite(CLOCK, HIGH);                  // pulse CLOCK
  digitalWrite(CLOCK, LOW);
 
  digitalWrite(DATA, LOW);                    // ensure DATA is LOW

  digitalWrite(LOAD, LOW);
  digitalWrite(LOAD, HIGH);                   // pulse LOAD
  digitalWrite(LOAD, LOW);

}

// setup additional program runtime parameters
void setup() {

  // setup Arduino DCR SDR radio record memory channel assignments (DDS ppm calibration calculated at 20C ambient air temperature & VDD = 4.95 volts)
  channel[ 0].label = "CALIBRATE";  channel[ 0].frequency = 20.0000e6;
  channel[ 1].label = "WSPR 20";    channel[ 1].frequency = 14.0956e6;
  channel[ 2].label = "WSPR 30";    channel[ 2].frequency = 10.1387e6;
  channel[ 3].label = "WSPR 40";    channel[ 3].frequency =  7.0386e6;
  channel[ 4].label = "CHU 20 ";    channel[ 4].frequency = 14.6700e6;
  channel[ 5].label = "CHU 40 ";    channel[ 5].frequency =  7.3900e6;
  channel[ 6].label = "CHU 80 ";    channel[ 6].frequency =  3.3300e6;
  channel[ 7].label = "NAVTEX";     channel[ 7].frequency =   .5163e6;
  channel[ 8].label = "VOARDOGRM1"; channel[ 8].frequency =  5.7450e6;
  channel[ 9].label = "VOARDOGRM2"; channel[ 9].frequency = 15.6700e6;
  channel[10].label = "VOARDOGRM3"; channel[10].frequency = 17.8700e6;
  channel[11].label = "FIST20";     channel[11].frequency = 14.0580e6;
  channel[12].label = "FIST30";     channel[12].frequency = 10.1180e6;
  channel[13].label = "FIST40";     channel[13].frequency =  7.0580e6;
  channel[14].label = "FIST80";     channel[14].frequency =  3.5580e6;
  channel[15].label = "ARRL20";     channel[15].frequency = 14.0475e6;
  channel[16].label = "ARRL40";     channel[16].frequency =  7.0475e6;
  channel[17].label = "ARRL80";     channel[17].frequency =  3.5815e6;
  channel[18].label = "WEFAX NMG";  channel[18].frequency =  4.3160e6;
  channel[19].label = "WEFAX NMG";  channel[19].frequency =  8.5020e6;
  channel[20].label = "WEFAX NMG";  channel[20].frequency = 12.7880e6;
  channel[21].label = "WEFAX NMF";  channel[21].frequency =  6.3886e6;
  channel[22].label = "WEFAX NMF";  channel[22].frequency =  9.1081e6;
  channel[23].label = "WWV 20";     channel[23].frequency = 15.0000e6;
  channel[24].label = "WWV 30";     channel[24].frequency = 10.0000e6;
  channel[25].label = "WWV 60";     channel[25].frequency =  5.0000e6;
  channel[26].label = "SSTV 20";    channel[26].frequency = 14.2300e6;

  pinMode (DATA,  OUTPUT);                   // set Arduino -> AD9850 DDS module assigned pins for output
    pinMode (CLOCK, OUTPUT);
  pinMode (LOAD,  OUTPUT); 
    pinMode (RESET, OUTPUT); 

  AD9850_init();                             // initialize & reset AD9850 DDS module
    AD9850_reset();

  // setup protoshield onboard pushbutton 'S1' for input & set status/state to logical HIGH or open/not pressed
  pinMode(BUTTON_S1, INPUT); 
    digitalWrite(BUTTON_S1, HIGH);
 
  index = readChannel();                     // get last saved radio memory channel from EEPROM (0 is default if nothing was stored previously)

  if(index < 0 || index > MAXCHANNELS)       // check for valid range (first time EEPROM use may have "junk" values stored)
    index = 0;

  lcd.init();                                // initialize LCD to default settings
    lcd.backlight();                         // enable LCD backlight (also can turn on/off using jumper on IIC display driver module)

} // end function setup()

// main Arduino DCR/SDR program begins here 
void loop() {
 
  do {
    
    // set current AD9850 DDS module frequency
    AD9850_setfreq(channel[index].frequency);

    // following displays results on LCD using IIC display module driver
    lcd.clear();                              // clear LCD display & set LCD cursor to line 1 character position 0

    if(tuning > 0 )
      lcd.print("+");
    else
      lcd.print("-");
    
    lcd.setCursor(2, 0);
    
    if(index < 10)
      lcd.print("0");
    
    lcd.print(index);
  
    lcd.setCursor(5, 0);
  
    if(channel[index].frequency < 10e6)
      lcd.print(" ");
  
    lcd.print(channel[index].frequency / 1e6, 4);
    lcd.print(" MHz");

    lcd.setCursor(0, 1);
    lcd.print(channel[index].label);
  
    while(1) {                            // loop forever until button S1 on protoboard pressed (or power removed)
      
      lcd.setCursor(11, 1);
    
      lcd.print("LO ");
      
      int LO_level = 0;
      
      for(int i = 0; i < 5; i++)
         LO_level += analogRead(RF_PROBE);
         
      if(LO_level < 100)
        lcd.print("lo");
      else
        if(LO_level > 350)
          lcd.print("hi");
      else
        lcd.print("ok");

      if(digitalRead(BUTTON_S1) == LOW) { // if button pressed break take required action
    
        delay(500);                       // delay for button debounce & check for long button press (changes memory channel tuning direction)
      
        if(digitalRead(BUTTON_S1) == LOW) {
         
          tuning *= -1;
 
          lcd.setCursor(0, 0);
          
          delay(500);
          
          break;
          
        }  
        else {
          index += tuning;
          
          if(index < 0)
            index = MAXCHANNELS;
          else
            if(index > MAXCHANNELS)
              index = 0;
        
          writeChannel(index);            // save channel index number to EEPROM
          
          break;                          // back to start of forever loop
        }
      }  
    }  

  } while(1); // end do loop
  
} // end loop() function

// function writes (saves) integer to EEPROM addresses 0 & 1
void writeChannel(int number) {
  
  EEPROM.write(0, highByte(number));
  EEPROM.write(1, lowByte(number));

}                                        // end writeChannel function

// function read integer from EEPROM address 0 & 1
int readChannel() {
  
  byte high = EEPROM.read(0);           // read high byte of integer
   byte low = EEPROM.read(1);           // read low byte of integer
   
  return (high << 8) + low;             // left shift (8-bit) high byte & add in low byte to reconstruct integer

}                                       // end readChannel function

